@@ -6,10 +6,13 @@
 ERR=0
 
 # Detect installed program type (fdupes or jdupes)
-ORIG_DUPE=false
-jdupes -v 2>/dev/null >/dev/null && ORIG_DUPE=jdupes
-fdupes -v 2>/dev/null >/dev/null && ORIG_DUPE=fdupes
-test ! -z "$WINDIR" && "$WINDIR/jdupes.exe" -v 2>/dev/null >/dev/null && ORIG_DUPE="$WINDIR/jdupes.exe"
+if [ -z "$ORIG_DUPE" ]
+	then ORIG_DUPE=false
+	jdupes -v 2>/dev/null >/dev/null && ORIG_DUPE=jdupes
+	fdupes -v 2>/dev/null >/dev/null && ORIG_DUPE=fdupes
+	test ! -z "$WINDIR" && "$WINDIR/jdupes.exe" -v 2>/dev/null >/dev/null && ORIG_DUPE="$WINDIR/jdupes.exe"
+fi
+
 if [ ! $ORIG_DUPE -v 2>/dev/null >/dev/null ]
 	then echo "Cannot run installed jdupes or fdupes"
 	exit 1
@@ -24,5 +27,13 @@ echo -en "\nBuilt jdupes:"
 sync
 time ./jdupes -nrq "$@" > built_output.txt || ERR=1
 diff -Nau installed_output.txt built_output.txt
+if [ -e jdupes-standalone ]
+	then
+	echo -en "\nBuilt jdupes-standalone:"
+	sync
+	time ./jdupes-standalone -nrq "$@" > built_output.txt || ERR=1
+	diff -Nau installed_output.txt built_output.txt
+fi
+
 rm -f installed_output.txt built_output.txt
 test "$ERR" != "0" && echo "Errors were returned during execution"
